"""Euchre card game module.

Trump-based trick-taking game with "going alone" and unique deck structure.
"""

from __future__ import annotations

from .game import EuchreGame

__all__ = ["EuchreGame"]
