"""Rummy 500 card game module.

Variant with melding, laying off, and negative scoring for cards in hand.
"""

from __future__ import annotations

from .game import Rummy500Game

__all__ = ["Rummy500Game"]
